import numpy as np
import matplotlib.pyplot as plt

### Elementary implementation for numerical integration of a neural field model
### u_t = -\alpha*u + \int_\Omega f(u(y)) M(x,y) dy for \Omega=[a,b], zero Neumann bc

# define the spatial mesh: total number of mesh points 'K+1', spacing 'kstep'
a = -80; b = 80; K = 400; kstep = (b-a)/K
xres=np.linspace(a,b,K+1)

# define the initial condition & linear damping parameter
uj=np.zeros(K+1)+0.8; alpha = 0.1

# define the nonlinearity
def vf( vec ):
    ## a sigmoid-type function
    #kappa=1.2; eta=0.2; F0=1.0;
    #res=F0/(1+np.exp(-eta*(vec-kappa)));
    ## a standard cubic
    res = (1+vec)*(1-vec)*(vec-0.1)
    return res

# build the M-matrix M(x_j,x_k) to have relavant kernel fct values
M=np.zeros((K+1,K+1)); sigma=0.05
for j in range(0,K):
    for k in range(0,K):
        #M[j,k]=1/2*np.exp(-abs(xres[j]-xres[k])) # Bressloff JPhysA p.5-6
        M[j,k]= ( 1/(sigma*np.sqrt(2*np.pi))
                *np.exp(-0.5/sigma*abs(xres[j]-xres[k])**2)) # approximate delta-fct
        
# define the sliced matrices for the trapezoidal rule
Mleft  = M[:,1:]; Mright = M[:,:K-1]
          
# define the time mesh, final time 'T' and stepsize 'tstep'
J=4*(10**5); T=2500.0; tstep = T/J

# define the noise
R = 4; Dt = R*tstep; L=int(J/R) # increment steps; L:=J/R
epsilon = 0.5                   # noise level
Nmat = np.eye(K+1,K+1)          # noise/correlation matrix 
dW = np.sqrt(tstep)*np.random.normal(size=(K+1,J))  # Brownian increments

# define the vectors and matrices for the output
ures = np.zeros((K+1,L)); ures[:,0] = uj
tres = np.linspace(0,T,L)

### main loop/iteration to march the solution forward, Euler/Milstein
for j in range(len(tres)-1):
    noise = epsilon* Nmat @ np.sum(dW[:,R*(j-1):R*j-1],1)
    fu = vf(uj)  # evaluate the nonlinearity
    drift = -alpha*uj + 0.5*kstep * ( Mleft @ fu[1:] + Mright @ fu[:K-1] )  
    uj = uj + Dt*drift + noise # main time iteration step
    uj[0]=uj[1]; uj[K]=uj[K-1] # Neumann bc 
    ures[:,j+1] = uj
    # plot the solution at certain time snapshots
    if np.mod(j,1000)<1:
        plt.clf()
        plt.plot(xres,uj,color='k',linewidth=0.5)
        plt.xlabel("x"); plt.ylabel("u(x,t)");
        plt.title("t=" + str(np.floor(tres[j])))
        plt.pause(0.0005)

plt.show()

# plot the solution as time snapshots
plt.clf()
plt.subplot(1, 3, 1)
plt.plot(xres, ures[:,0],color='k',linewidth=1)
axes = plt.gca(); 
axes.set_xlim([a,b]); axes.set_ylim([-1.4,1.4])
axes.xaxis.set_major_formatter(plt.NullFormatter())
axes.yaxis.set_major_formatter(plt.NullFormatter())

plt.subplot(1, 3, 2)
plt.plot(xres, ures[:,int(np.floor(L/2))],color='k',linewidth=1)
axes = plt.gca(); 
axes.set_xlim([a,b]); axes.set_ylim([-1.4,1.4])
axes.xaxis.set_major_formatter(plt.NullFormatter())
axes.yaxis.set_major_formatter(plt.NullFormatter())


plt.subplot(1, 3, 3)
plt.plot(xres, ures[:,K-1],color='k',linewidth=1)
axes = plt.gca(); 
axes.set_xlim([a,b]); axes.set_ylim([-1.4,1.4])
axes.xaxis.set_major_formatter(plt.NullFormatter())
axes.yaxis.set_major_formatter(plt.NullFormatter())

# re-size a bit for saving it properly
#figure = plt.gcf()
#figure.set_size_inches(4, 2)
plt.savefig('fig01.eps', bbox_inches='tight')

plt.show()


